#!/usr/bin/env python3
# @Time    : 17-9-10 01:08
# @Author  : Wavky Huang
# @Contact : master@wavky.com
# @File    : meta.py

"""

"""
PROJECT_NAME = 'ManHourCalendar'
VERSION = '0.9.1b2'
AUTHOR = 'Wavky Huang'
AUTHOR_EMAIL = 'master@wavky.com'
